#![no_std]
#![no_main]

// The macro for our start-up function
use rp_pico::entry;

// GPIO traits
use embedded_hal::digital::v2::OutputPin;

use defmt as _;
use defmt_rtt as _;
use panic_probe as _;

// Pull in any important traits
use rp_pico::hal::prelude::*;

// A shorter alias for the Peripheral Access Crate, which provides low-level
// register access
use rp_pico::hal::pac;

// A shorter alias for the Hardware Abstraction Layer, which provides
// higher-level drivers.
use rp_pico::hal;

/// Entry point to our bare-metal application.
///
/// The `#[entry]` macro ensures the Cortex-M start-up code calls this function
/// as soon as all global variables are initialised.
#[entry]
fn main() -> ! {
    // Grab our singleton objects
    let mut pac = pac::Peripherals::take().unwrap();
    let core = pac::CorePeripherals::take().unwrap();

    // Set up the watchdog driver - needed by the clock setup code
    let mut watchdog = hal::Watchdog::new(pac.WATCHDOG);

    // Configure the clocks
    //
    // The default is to generate a 125 MHz system clock
    let clocks = hal::clocks::init_clocks_and_plls(
        rp_pico::XOSC_CRYSTAL_FREQ,
        pac.XOSC,
        pac.CLOCKS,
        pac.PLL_SYS,
        pac.PLL_USB,
        &mut pac.RESETS,
        &mut watchdog,
    )
    .ok()
    .unwrap();

    // The delay object lets us wait for specified amounts of time (in
    // milliseconds)
    let mut delay = cortex_m::delay::Delay::new(core.SYST, clocks.system_clock.freq().to_Hz());

    // The single-cycle I/O block controls our GPIO pins
    let sio = hal::Sio::new(pac.SIO);

    // Set the pins up according to their function on this particular board
    let pins = rp_pico::Pins::new(
        pac.IO_BANK0,
        pac.PADS_BANK0,
        sio.gpio_bank0,
        &mut pac.RESETS,
    );

    let mut panel_a = pins.gpio2.into_push_pull_output();
    let mut panel_b = pins.gpio3.into_push_pull_output();
    let mut panel_oe = pins.gpio4.into_push_pull_output();
    let mut panel_lat = pins.gpio5.into_push_pull_output();
    let mut panel_clk = pins.gpio6.into_push_pull_output();
    let mut panel_c = pins.gpio7.into_push_pull_output();
    let mut panel_d = pins.gpio8.into_push_pull_output();
    let mut panel_e = pins.gpio9.into_push_pull_output();
    let mut panel_r1 = pins.gpio10.into_push_pull_output();
    let mut panel_g1 = pins.gpio11.into_push_pull_output();
    let mut panel_b1 = pins.gpio12.into_push_pull_output();

    // This doesn't seem to work yet...
    panel_oe.set_high().unwrap();
    panel_a.set_high().unwrap();
    panel_b.set_high().unwrap();
    panel_c.set_high().unwrap();
    panel_d.set_high().unwrap();
    panel_e.set_high().unwrap();
    loop {
        for _ in 0..32 {
            panel_r1.set_high().unwrap();
            panel_g1.set_high().unwrap();
            panel_b1.set_high().unwrap();
            panel_clk.set_high().unwrap();
            delay.delay_us(2);
            panel_clk.set_low().unwrap();
            delay.delay_us(2);
        }
        panel_lat.set_high().unwrap();
        delay.delay_us(2);
        panel_lat.set_low().unwrap();
        delay.delay_us(2);
    }
}
